# Package:  autodock
# Date:     15th November 2014
# Author:   James Mills, prologic at shortcircuit dot net dot au


"""autodock - Daemon for Docker Automation

autodock is a Daemon for Automating Docker.

:copyright: CopyRight (C) 2014 by James Mills
"""


__author__ = "James Mills, prologic at shortcircuit dot net dot au"
__date__ = "15th November 2014"


from .version import version as __version__  # noqa
